*Euclidean space tools
*SPSS macros by Kirill Orlov
*kior@comtv.ru
*http://www.spsstools.net/KO-spssmacros.htm
*All rights reserved.

*/*!SPTOD*/***************************************************************************************.
define !sptod(cols= !charend('/') /dist= !charend('/'))
set results none.
flip vari= !cols /newname= varname_.
set results list.
matrix.
get m /vari= !cols /names= names.
do if any(m<>t(m)).
-print /title ">WARNING. Input matrix isn't fully symmetrical by values.".
-print /title ' Symmetry restored by averaging symmetrical elements of the matrix.' /space= 0.
-comp m= (m+t(m))/2.
end if.
print (eval(m)*ncol(m)/trace(m)) /title= 'Eigenvalues of the input matrix (rescaled to mean=1):'.
comp hsq= kroneker(diag(m),make(1,ncol(m),1)).
comp m= hsq+t(hsq)-2*m.
do if any(m<0).
-print /title '>WARNING. Configuration is so non-euclidean that some square distances <0.'.
 !if (!dist<>'SEUCLID') !then
- print /title ' The root is extracted of them like of positive, and negative sign retained.' /space= 0.
 !ifend
end if.
!if (!dist<>'SEUCLID') !then
-comp m= (-(m<0)+(m>=0))&*sqrt(abs(m)).
!ifend
comp m= {make(ncol(m),1,'PROX'),t(names),m}.
comp names= {'rowtype_','varname_',names}.
save m /outfile= * /names= names /string= rowtype_ varname_.
end matrix.
!if (!dist=!null) !then !let !dst= 'EUCLID' !else !let !dst= !dist !ifend
val lab rowtype_ 'PROX' !quote(!conc('DISSIMILARITY ',!dst)).
!enddefine.

*/*!DTOSP*/***************************************************************************************.
define !dtosp(cols= !charend('/') /square= !charend('/') /diag= !charend('/'))
set results none.
flip vari= !cols /newname= varname_.
set results list.
matrix.
get m /vari= !cols /names= names.
do if any(diag(m))<>0).
-print /title '>WARNING. Input matrix has nonzero elements on the diagonal.'.
-print /title ' They were replaced by zero.' /space= 0.
-call setdiag(m,0).
end if.
do if any(m<>t(m)).
-print /title ">WARNING. Input matrix isn't fully symmetrical by values.".
-print /title ' Symmetry restored by averaging symmetrical elements of the matrix.' /space= 0.
-comp m= (m+t(m))/2.
end if.
do if any(m<0).
-print /title '>WARNING. There are negative distances in the input matrix.'.
 !if (!square<>'NO') !then
- print /title ' Negative sign retained for them at raising to square.' /space= 0.
 !ifend
end if.
!if (!square<>'NO') !then
-comp m= (-(m<0)+(m>=0))&*m&**2.
!ifend
!if (!diag='CENTER' !or !diag=!null) !then
-comp rmean= kroneker(rsum(m)/ncol(m),make(1,ncol(m),1)).
-comp m= (rmean+t(rmean)-m-msum(m)/ncol(m)**2)/2.
!else
-get hsq /var= ss.
-comp hsq= kroneker(hsq,make(1,ncol(m),1)).
-comp m= (hsq+t(hsq)-m)/2.
!ifend
print (eval(m)*ncol(m)/trace(m)) /title= 'Eigenvalues of the output matrix (rescaled to mean=1):'.
comp m= {make(ncol(m),1,'PROX'),t(names),m}.
comp names= {'rowtype_','varname_',names}.
save m /outfile= * /names= names /string= rowtype_ varname_.
end matrix.
val lab rowtype_ 'PROX' 'SIMILARITY SSCP'.
!enddefine.

*/*!SPTOEUC*/***************************************************************************************.
define !sptoeuc(cols= !charend('/') /k= !charend('/') /elong= !charend('/') /return= !charend('/') /mxiter= !charend('/') !default(500))
set mxloops !mxiter results none.
flip vari= !cols /newname= varname_.
set results list.
matrix.
get sp /vari= !cols /names= names.
do if any(sp<>t(sp)).
-print /title ">WARNING. Input matrix isn't fully symmetrical by values.".
-print /title ' Symmetry restored by averaging symmetrical elements of the matrix.' /space= 0.
-comp sp= (sp+t(sp))/2.
end if.
comp oldtr= trace(sp).
comp eival= eval(sp).
print {eival,(eival*ncol(sp)/trace(sp))} /title= 'Original eigenvalues (right - rescaled to mean=1):' /format= f20.15.
do if eival(nrow(eival))>=0.
-print /title= 'No iterations needed: input SSCP matrix is euclidean.'. 
-print /title= 'It was output in a new data window.'.
else.
 !if (!k=!null) !then
- comp k= csum(diag(sp))/ncol(sp)/100.
 !else
- comp k= !k.
 !ifend
-loop i= 1 to !mxiter.
  !if (!elong='H') !then
-  comp h= kroneker(sqrt(diag(sp)),make(1,ncol(sp),1)).
-  comp sp= !k*(h+t(h)+!k)+sp.
  !else
-  comp sp= !k+sp.
  !ifend
- comp eival= eval(sp).
-end loop if eival(nrow(eival))>=0.
-print i /title= ' :'.
-print {eival,(eival*ncol(sp)/trace(sp))} /title= 'Achieved eigenvalues (right - rescaled to mean=1):' /format= f20.15.
-do if eival(nrow(eival))<0.
- print /title= '>WARNING. Euclidness not achieved. Try to raise MXITER or K.'.
  !if (!elong='H') !then
-  print /title= "  If it doesn't help try ELONG=HSQ." /space= 0.
  !else
-  print /title= "  If doesn't help, non-euclidean are distances b/w points, not diagonal is." /space= 0.
  !ifend
-end if.
-print /title= 'Altered SSCP matrix was output in a new data window.'.
-!if (!return='YES') !then
- comp sp= sp*oldtr/trace(sp).
- print /title= 'It was recalculated so that sum of diagonal elements become original.'.
- comp eival= eval(sp).
- print {eival,(eival*ncol(sp)/trace(sp))} /title= 'Its eigenvalues (right - rescaled to mean=1):' /format= f20.15.
-!ifend
end if.
comp sp= {make(ncol(sp),1,'PROX'),t(names),sp}.
comp names= {'rowtype_','varname_',names}.
save sp /outfile= * /names= names /string= rowtype_ varname_.
end matrix.
val lab rowtype_ 'PROX' 'SIMILARITY SSCP'.
!enddefine.

*/*!DTOEUC*/***************************************************************************************.
define !dtoeuc(cols= !charend('/') /k= !charend('/') /mxiter= !charend('/') !default(500))
set mxloops !mxiter results none.
flip vari= !cols /newname= varname_.
set results list.
matrix.
get d /vari= !cols /names= names.
do if any(d<>t(d)).
-print /title ">WARNING. Input matrix isn't fully symmetrical by values.".
-print /title ' Symmetry restored by averaging symmetrical elements of the matrix.' /space= 0.
-comp d= (d+t(d))/2.
end if.
comp rmean= kroneker(rsum(d&**2)/ncol(d),make(1,ncol(d),1)).
comp eival= eval((rmean+t(rmean)-d&**2-msum(d&**2)/ncol(d)**2)/2).
print {eival,eival*ncol(d)/csum(eival)} /title= 'Original eigenvalues (right - rescaled to mean=1):' /format= f20.15.
do if eival(nrow(eival))>=0.
-print /title= 'No iterations needed: input matrix of distances is euclidean.'. 
-print /title= 'It was output in a new data window.'.
else.
 !if (!k=!null) !then
- comp k= msum(d)/(ncol(d)**2-ncol(d))/100.
 !else
- comp k= !k.
 !ifend
-loop i= 1 to !mxiter.
- comp d= d+k.
- call setdiag(d,0).
- comp rmean= kroneker(rsum(d&**2)/ncol(d),make(1,ncol(d),1)).
- comp eival= eval((rmean+t(rmean)-d&**2-msum(d&**2)/ncol(d)**2)/2).
-end loop if eival(nrow(eival))>=0.
-print i /title= 'Iterations needed:'.
-print {eival,eival*ncol(d)/csum(eival)} /title= 'Achieved eigenvalues (right - rescaled to mean=1):' /format= f20.15.
-do if eival(nrow(eival))<0.
- print /title= '>WARNING. Euclidness not achieved. Try to raise MXITER or K.'.
-end if.
-print /title= 'Altered matrix of distances was output in a new data window.'.
end if.
comp d= {make(ncol(d),1,'PROX'),t(names),d}.
comp names= {'rowtype_','varname_',names}.
save d /outfile= * /names= names /string= rowtype_ varname_.
end matrix.
val lab rowtype_ 'PROX' 'DISSIMILARITY EUCLID'.
!enddefine.
